
#    Note:  This is loaded into the namespace  FEBEX_TriggerMatrix

# variable started 0

if {![info exists started]} {variable started 0}

global Options
global env

if {$started != 0} {

     variable JS ""
     variable LogFlag

    if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1

    variable LogText ""
    variable LogFlag 0
    variable LogCount 0

    variable ShowLogFlag 0
    variable DebugFlag 0
    
    global ModuleMenu
    global LinkMenu
    global BoardMenu  
  
    global FEBEXmodule
    global FEBEXlink
    global FEBEXboard
    
    set FEBEXmodule 0
    set FEBEXlink 0
    set FEBEXboard 0
    
    variable allmodules 0
    variable alllinks 0
    variable allboards 0
    
    variable Window 0

    variable JS ""

    variable DBNameList {"xxx"}
    
    variable AcqSetOption ""
   
   for {set k 17} {$k >= 0} {incr k -1} {
         variable RD.$k; set RD.$k 0
      for {set i 16} {$i >= 0} {incr i -1} {
         variable RD.$k.$i; set RD.$k.$i 0
      }
   }
   
    variable A01 0
    variable A02 0
    
     set ::FEBEX::DBNodeBase $env(ComputerName)


   proc show_variables {} {
        global MIDAS_DataAcquisitionControlServer_Exists
        global Options
        variable LogFlag
        variable ShowLogFlag
        variable AcqSetOption
        variable Window

        InsertLog "MIDAS_DataAcquisitionControlServer_Exists=$MIDAS_DataAcquisitionControlServer_Exists"

        InsertLog "LogFlag=$LogFlag; ShowLogFlag=$ShowLogFlag"
        InsertLog "DBPath = $::FEBEX::DBRootName  $::FEBEX::DBNodeBase  $::FEBEX::DBNodeName"
        InsertLog "AcqSetOption = $AcqSetOption"
        InsertLog "Window = $Window"

   }
   
   proc UpdateModule {} {}

    proc do_setopt {option} {
        global errorCode errorInfo
        global DataAcquisitionControlServerHost
        variable AcqSetOption
        global MIDAS_DataAcquisitionControlServer_Exists

        set AcqSetOption $option

       if {[info exists MIDAS_DataAcquisitionControlServer_Exists] && $MIDAS_DataAcquisitionControlServer_Exists == 1} {

          set rc [catch {interp eval "" ::namespace inscope ::urn:DataAcquisitionControlServer DoOption "$AcqSetOption"} m]
       } else {

           if {$DataAcquisitionControlServerHost != $::DataAcquisition::ACQSERVER} {
                set DataAcquisitionControlServerHost  $::DataAcquisition::ACQSERVER
                DataAcquisitionControlClient_Configure
           }
           set rc [catch {DataAcquisitionControlClient__DoOption "$AcqSetOption"} m]
      }

        if {$rc != 0} {
            InsertLog "DO OPTION returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
#            InsertLog "DO OPTION returned $m ([format "0x%x" $m])"
        }

     }
     
      proc do_Save {} {
        global Options
      
        InsertLog "Saving FEBEX settings to $::FEBEX::DBRootName/$::FEBEX::DBNodeBase/$::FEBEX::DBNodeName"

        do_setopt "ParameterSave=$::FEBEX::DBNodeName"
        
        set Options(FirmWare.Settings) $::FEBEX::DBNodeName
        
        InsertLog "Done at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

        do_DBUpdate
    }

      proc do_Restore {} {
        global Options
      
        InsertLog "Restoring FEBEX settings from $::FEBEX::DBRootName/$::FEBEX::DBNodeBase/$Options(FirmWare.Settings)"

        do_setopt "ParameterRestore=$Options(FirmWare.Settings)"
         
        InsertLog "Done at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"
       
        do_DBUpdate
    }
    
      proc do_DBName {vl} {
         global Options

          set ::FEBEX::DBNodeName $vl
          DBCreateNode $::FEBEX::DBRootName/$::FEBEX::DBNodeBase/$::FEBEX::DBNodeName
          set Options(FirmWare.Settings) $vl
          SaveOptions

          InsertLog "Data Base node $::FEBEX::DBRootName/$::FEBEX::DBNodeBase contains [DBGetNodes $::FEBEX::DBRootName/$::FEBEX::DBNodeBase *]"
      }

      proc do_DBUpdate {} {
          variable DBNameList

          set DBNameList {}
          foreach item [DBGetNodes $::FEBEX::DBRootName/$::FEBEX::DBNodeBase *] {
              if {$item == "Options"} continue
              lappend DBNameList $item
          }
          if {$::FEBEX::DBNodeBase == "undefined"} {set ::FEBEX::DBNodeBase [first $DBNameList]}
      }
    
    proc do_Apply {R D vl} {
       global FEBEXlink
       global FEBEXboard
    
       InsertLog "do_Apply $R $D $vl"
       
       for {set m 17} {$m >= 0} {incr m -1} {
             variable RD.$m
          for {set n 16} {$n >= 0} {incr n -1} {
             variable RD.$m.$n
          }
       }
       
       set RD.$R.$D [expr ($vl + 1) & 1]
       
       set RD.$R 0
       for {set n 16} {$n >=0} {incr n -1} {
           set RD.$R [format "0x%08x" [expr [set RD.$R] | ([set RD.$R.$n] << $n)]]
       }
       
       InsertLog "RD.$R  => [set RD.$R]"
       
       WriteFEBEX4ARegister $FEBEXlink $FEBEXboard $R tm_row [set RD.$R]
       
       InsertLog "Write TriggerMatrix [set RD.$R]; Read [ReadFEBEX4ARegister $FEBEXlink $FEBEXboard $R tm_row]"
    
     }
     
   proc TriggerWindow {} {
       global FEBEXlink
       global FEBEXboard
       variable Window
       
        WriteFEBEX4ARegister $FEBEXlink $FEBEXboard 0 tm_window $Window
       
       InsertLog "Write TriggerWindow [set Window]; Read [ReadFEBEX4ARegister $FEBEXlink $FEBEXboard 0 tm_window]"      
       
   }
     
    proc do_Setting {p vl} {
        variable DebugFlag
        variable Window
    
    if {$DebugFlag != 0}  {InsertLog "do_Setting $p = $vl"}
    
        switch -glob $p {
           Choice*      {set p [string range $p 6 end]}
           rangeInputS* {set p [string range $p 11 end]}
           textInputS*  {set p [string range $p 10 end]}
           Text*        {set p [string range $p 4 end]}
        }
        
        switch -glob $p {
            WIN  {set Window $vl; TriggerWindow}
            
            default  {InsertLog "unexpected Setting:  $p"}
        }
        
    }

   proc  do_click {} {
       variable cgidata
       variable started
       variable LogFlag
       variable DebugFlag
       variable ShowLogFlag
       variable allmodules
       variable alllinks
       variable allboards

      set cgidata [ncgi::nvlist]

      if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}

      if {[llength $cgidata] == 0} then {return}

      if {[llength $cgidata] < 2} then {
         InsertLog "CGI data is too short: $cgidata"
      } else {
        if {[lindex $cgidata 0] != "Widget"} then {
            InsertLog "Invalid CGI data: $cgidata"
        } else {
           set w [lindex $cgidata 1]
           set i [second [lindex $cgidata 1]]
           set vl [third [lindex $cgidata 1]]

           switch -glob [lindex $w 0] {

                RESET    {set started 0}
                CLEAR    {EmptyLog}
                ELOG     {PrintLog}
                RELOAD   {}
                VARIABLES {show_variables}

                HIDE     {set ShowLogFlag 0}
                SHOW     {set ShowLogFlag 1}
                LOGON    {set LogFlag 1}
                LOGOFF   {set LogFlag 0}
                DEBUGON    {set DebugFlag 1}
                DEBUGOFF   {set DebugFlag 0}
                   
   		HELP     {variable HelpTopic; set HelpTopic Help; do_help}  
   
                MODULE     {SelectFEBEXModule $i; UpdateModule}
                ALLMODULES {set allmodules [expr ($allmodules + 1) & 1]}
                LINK     {SelectFEBEXLink $i; UpdateModule}
                ALLLINKS {set alllinks [expr ($alllinks + 1) & 1]}
                BOARD     {SelectFEBEXBoard $i; UpdateModule}
                ALLBOARDS {set allboards [expr ($allboards + 1) & 1]}
                   
                RANGE -
                TEXT -
                SETTINGS  {do_Setting $i $vl}       
   	
                DBNAMES -
                DBNAME   {do_DBName [lindex $w 1]}
                
                APPLY    {do_Apply [lindex $w 1] [lindex $w 2] [lindex $w 3]}

                GENKEY   {do_DBName [date +%Y%h%d-%H.%M.%S]}

                RESTORE      {do_Restore}
                RESTORESTATE {do_RestoreState}
                SAVE         {do_Save}
                SAVESTATE    {do_SaveState}
      
                undefined {}
                default  {InsertLog "unexpected cgi data:  $cgidata"}
            }
         }
       }
       
       do_DBUpdate
    }

     do_DBUpdate
     LoadOptions


     set html ""


}  ;# end of if {$started == 0}

